/* 
 * File:   configuration.h
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Firmware configuration data.
 */

#ifndef __CONFIGURATION_H
#define	__CONFIGURATION_H

#include "mcc_generated_files/mcc.h"

// Automatic rescue timeout and DUL-related alarm enabling depends on the KM3Net site
#if defined(ARCA_CONFIGURATION)
    // used in VERSION command
    #define CONFIG_FIRMWARE_TARGET 0x00000001
    // ARCA default rescue timeout in seconds: 25 minutes
    #define RESCUE_TIMEOUT_SECONDS (25*60)
    // ARCA DU has the DUL, so related alarms must be enabled
    #define DEFAULT_FLAG_DUL_ALARMPOS1_ALARM_ENABLED true
    #define DEFAULT_FLAG_DUL_ALARMPOS2_ALARM_ENABLED true
    #define DEFAULT_FLAG_DUL_ALARMNEG1_ALARM_ENABLED true
    #define DEFAULT_FLAG_DUL_ALARMNEG2_ALARM_ENABLED true
#elif defined(ORCA_CONFIGURATION)
    // used in VERSION command
    #define CONFIG_FIRMWARE_TARGET 'ORCA'
    // ORCA default rescue timeout in seconds: 15 minutes
    #define RESCUE_TIMEOUT_SECONDS (15*60)
    // ORCA DU has NOT the DUL, so related alarms must be disabled
    #define DEFAULT_FLAG_DUL_ALARMPOS1_ALARM_ENABLED false
    #define DEFAULT_FLAG_DUL_ALARMPOS2_ALARM_ENABLED false
    #define DEFAULT_FLAG_DUL_ALARMNEG1_ALARM_ENABLED false
    #define DEFAULT_FLAG_DUL_ALARMNEG2_ALARM_ENABLED false
#else
    #error "Configuration undefined (ARCA_CONFIGURATION or ORCA_CONFIGURATION macros undefined)."
#endif

// time between rescue action steps in seconds: 1 minutes
#define RESCUE_STEP_INTERVAL_SECONDS 60

// period of timers SLOW and FAST in microseconds
#define IRQ_TIMER_SLOW_PERIOD_US 800 // from what MCC reports
#define IRQ_TIMER_FAST_PERIOD_US 200 // from what MCC reports

// dimension of the buffer for the payload of the packets
#define COMMUNICATION_DATABUF_SIZE 240

// adcreader-related configuration parameters
#define ADCREADER_INIT_TIMEOUT 64                   // number of timer ticks to wait after start
#define ADCREADER_CONVSTART_TIMEOUT 1               // number of timer ticks from mux change to start of conversion
#define ADCREADER_MUXCHANGE_TIMEOUT 1               // number of timer ticks from start of conversion to next mux change
#define ADCREADER_AVERAGING_PRESCALER_DEFAULT 10    // prescaler value for sensor reading averaging (e.g. 8 => sum 256 values)

#endif// __CONFIGURATION_H

