/*
 * File:   irqs.c
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Interrupt service routines.
 * 
 */

#include "irqs.h"
#include "adcreader.h"
#include "generated/sources/variables.h"
#include "timing.h"

// local functions
void _manage_digital_variables(void);
void _manage_digital_variable(digital_variable_t* _p_variable);

// "slow" timer interrupt service routine
void irq_timer_slow(void) {  
        
    // __TODO LED remove user pin activity
    // userpin_SWITCH_LED_D9_set_high();    
    
    // update the adcreader state machine
    adcreader_tick_irq(); 
    
    // update timing register
    timing_tick_irq();
        
    // __TODO LED remove user pin activity
    // userpin_SWITCH_LED_D9_set_low();     
}

// "fast" timer interrupt service routine
void irq_timer_fast(void) {   
        
    // __TODO LED remove user pin activity 
    // userpin_SWITCH_LED_D10_set_high();     
    
    // manage digital alarms
    _manage_digital_variables();  
        
    // __TODO LED remove user pin activity
    // userpin_SWITCH_LED_D10_set_low();    
    
}

void _manage_digital_variables(void) {    
    for(uint8_t idx = 0; idx < DIGITAL_VARIABLES_COUNT; idx++) {
        _manage_digital_variable(&(digital_variables[idx]));
    }    
}

void _manage_digital_variable(digital_variable_t* _p_variable) {
    
    // read digital variables
    _p_variable->value = _p_variable->readfunc();

    // and update value_max
    if(_p_variable->value > _p_variable->value_max) {
        _p_variable->value_max = _p_variable->value;
    }
    
    // skip alarm management if alarm not implemented on current variable
    if(_p_variable->alarm.implement) {
        // check if timout counter has to be incremented
        if(_p_variable->value > 0) {           
            // increment counter
            _p_variable->alarm.timeout_counter += 1;
            // avoid wrapping on increment (not very elegant...)
            if(_p_variable->alarm.timeout_counter == 0) {
                _p_variable->alarm.timeout_counter -= 1;
            }
            // update alarm counter stats
            if(_p_variable->alarm.timeout_counter_max < _p_variable->alarm.timeout_counter) {
                _p_variable->alarm.timeout_counter_max = _p_variable->alarm.timeout_counter;
            }
        }else{
            // or reset the counter
            _p_variable->alarm.timeout_counter = 0;
        }
        // check if alarm has fired
        if(_p_variable->alarm.enabled) {
            // check if timeout was reached
            if(_p_variable->alarm.timeout_counter > _p_variable->alarm.timeout) {
                // perform the alarm action
                _p_variable->alarm.callback();
                // increment the fire counter
                _p_variable->alarm.firecount += 1;
                // avoid wrapping on increment (not very elegant...)
                if(_p_variable->alarm.firecount == 0) {
                    _p_variable->alarm.firecount -= 1;
                }
                // reset alarm timeout counter
                _p_variable->alarm.timeout_counter = 0;
            }
        }
    }
    
}
