/* 
 * File:   typedefs.h
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Type definitions.
 * 
 */

#ifndef __TYPEDEFS_H
#define	__TYPEDEFS_H

#include <stdint.h>
#include <stdbool.h>

#include "mcc_generated_files/adc.h"

// error value type definition
typedef uint8_t err_t;

// commands related types
typedef uint8_t command_code_t;             // command code
typedef uint8_t command_datalen_t;          // command data length
typedef uint8_t command_index_t;            // command counting index
typedef err_t (*command_callback_t)(void);  // command function pointer

// command info type definition
typedef struct {
    command_code_t request_code;            // command request code
    command_code_t response_code;           // command response code
    command_datalen_t request_payload_len;  // expected number of bytes in payload data
    command_callback_t callback_ptr;        // function pointer to the code to execute    
} command_info_t;

// ADC related types
typedef adc_result_t analog_value_t;
typedef adc_channel_t analog_channel_t;

// analog variables average related types
typedef uint32_t analog_variable_sum_t;
typedef uint16_t analog_variable_sum_counter_t;

// digital variables related types
typedef uint8_t digital_value_t;                            // maybe bool could be used
typedef digital_value_t (*digital_value_readfunc_t)(void);  // pointer function for reading one digital value

// alarm related types
typedef uint16_t alarm_timeout_t;
typedef void (*alarm_callback_t)(void);
typedef uint16_t alarm_firecount_t;

// single analog alarm info definition
typedef struct {
    bool implement;                         // configuration variable to implement the alarm or not
    bool enabled;                           // configuration variable to enable or disable the alarm
    analog_value_t threshold;               // analog alarm configuration threshold value
    alarm_timeout_t timeout;                // analog alarm timeout configuration value (in number of "ticks")
    alarm_timeout_t timeout_counter;        // variable used to count, will be compared to timeout    
    alarm_timeout_t timeout_counter_max;    // variable used store the maximum count value
    alarm_firecount_t firecount;            // counts how many times the alarm has fired
} analogalarm_info_t;

// dual (low- and high-threshold) analog alarm info definition
typedef struct {
    analogalarm_info_t fast;
    analogalarm_info_t slow;
    alarm_callback_t callback; // function called when alarm fires
} analogalarm_dual_info_t;

// digital alarm info definition
typedef struct {
    bool implement;
    bool enabled; 
    alarm_timeout_t timeout; 
    alarm_timeout_t timeout_counter; 
    alarm_timeout_t timeout_counter_max;
    alarm_callback_t callback;  
    alarm_firecount_t firecount;
} digitalalarm_info_t;

// also include auto-generated types
#include "generated/sources/datatypes.h"

#endif	// __TYPEDEFS_H

